# coding: utf-8
from __future__ import unicode_literals
import codecs

STATUS_SUCCESS = 0
STATUS_PERMISSION_DENIED = 1
STATUS_BOT_EXCEPTION = 2
STATUS_TORNADO_EXCEPTION = 3
STATUS_ERROR = 4

DEFAULT_REMIND_TIME = 60 * 60
DEFAULT_REPORT_TIME = 60 * 60 * 1000

DELAY_TASK = 'DELAY_TASK'
PERIODIC_TASK = 'PERIODIC_TASK'
SYSTEM_TASK = 'SYSTEM_TASK'

MESSAGE_REPORT_COMMAND = '@wsr'
MESSAGE_STATUS_COMMAND = '@wss'


def _read_config_list():
    with codecs.open('conf.ini', 'w+', encoding='utf-8') as f1:
        conf_list = [conf for conf in f1.read().split('\n') if conf != '']
        return conf_list


def write_config(name, value):
    name = name.lower()
    new = True
    conf_list = _read_config_list()
    for i, conf in enumerate(conf_list):
        if conf.startswith(name):
            conf_list[i] = '{0}={1}'.format(name, value)
            new = False
            break
    if new:
        conf_list.append('{0}={1}'.format(name, value))

    with codecs.open('conf.ini', 'w+', encoding='utf-8') as f1:
        for conf in conf_list:
            f1.write(conf + '\n')
    return True


def read_config(name):
    name = name.lower()
    conf_list = _read_config_list()
    for conf in conf_list:
        if conf.startswith(name):
            return conf.split('=')[1].split('#')[0].strip()
    return None


class StatusWrapperMixin(object):
    status_code = STATUS_SUCCESS
    status_message = ''

    def write(self, chunk):
        context = {'status': self.status_code,
                   'message': chunk}
        super(StatusWrapperMixin, self).write(context)



# coding: utf-8
from __future__ import unicode_literals


class WxBot(object):
    def __new__(cls, *args, **kwargs):
        if not hasattr(cls, '_instance'):
            orig = super(WxBot, cls)
            cls._instance = orig.__new__(cls, *args, **kwargs)
        return cls._instance

    def __init__(self, bot=None, receiver=None, *args, **kwargs):
        self.bot = bot
        if not receiver:
            receiver = self.bot.file_helper
        self.receiver = receiver
        super(WxBot, self).__init__(*args, **kwargs)

    def send_msg(self, msg):
        self.receiver.send_msg(msg)


class Message(object):
    def __init__(self, content, title=None, time=None, remind=None, interval=None):
        self.title = title
        self.content = content
        self.message_time = time
        self.remind_time = None
        if time and remind:
            self.remind_time = time - remind
        self.nc = remind
        self.message_interval = interval

    @property
    def time(self):
        return self.message_time.strftime('%Y-%m-%d %H:%M:%S') if self.message_time else None

    @property
    def interval(self):
        return '{0}s'.format(self.message_interval.seconds) if self.message_interval else None

    @property
    def remind(self):
        return self.remind_time.strftime('%Y-%m-%d %H:%M:%S') if self.message_time else None

    def render_message(self):
        message = None
        if self.title:
            message = '标题：{0}'.format(self.title)
        if self.message_time:
            message = '{0}\n时间：{1}'.format(message, self.time)
        if message:
            message = '{0}\n内容：{1}'.format(message, self.content)
        else:
            message = self.content
        return message

    def __repr__(self):
        return self.render_message()


class Global(object):
    def __new__(cls, *args, **kwargs):
        if not hasattr(cls, '_instance'):
            orig = super(Global, cls)
            cls._instance = orig.__new__(cls, *args, **kwargs)
        return cls._instance

    def __init__(self, *args, **kwargs):
        for k, v in kwargs.iteritems():
            setattr(self, k, v)

    def insert(self, name, value):
        setattr(self, name, value)
        return True

    def __call__(self, *args, **kwargs):
        return self.__dict__



# coding: utf-8
# from __future__ import unicode_literals
import codecs
import os

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


def read(fname):
    return codecs.open(os.path.join(os.path.dirname(__file__), fname), encoding='utf-8').read()


NAME = "wechat_sender"

PACKAGES = ["wechat_sender", ]

DESCRIPTION = "将程序运行结果及报警信息通过微信发送给你自己"

LONG_DESCRIPTION = read("README.rst")

KEYWORDS = ["Wechat", "微信", "监控"]

AUTHOR = "RaPoSpectre"

AUTHOR_EMAIL = "rapospectre@gmail.com"

URL = "https://github.com/bluedazzle/wechat_sender"

VERSION = "0.0.3"

LICENSE = "BSD"

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    classifiers=[
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Operating System :: OS Independent',
        'Topic :: Communications :: Chat',
        'Topic :: Utilities',
    ],
    install_requires=[
        'tornado',
    ],

    keywords=KEYWORDS,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url=URL,
    license=LICENSE,
    packages=PACKAGES,
    include_package_data=True,
    zip_safe=True,
)



# coding: utf-8
from __future__ import unicode_literals

import copy
import functools
import json
import os
import time
import datetime
import psutil
import logging

import sys
import tornado.web

from objects import WxBot, Message, Global
from wechat_sender.utils import StatusWrapperMixin, STATUS_BOT_EXCEPTION, STATUS_PERMISSION_DENIED, \
    STATUS_TORNADO_EXCEPTION, DEFAULT_REMIND_TIME, STATUS_ERROR, DEFAULT_REPORT_TIME, DELAY_TASK, PERIODIC_TASK, \
    MESSAGE_REPORT_COMMAND, SYSTEM_TASK, MESSAGE_STATUS_COMMAND

glb = None
_logger = logging.getLogger(__name__)


class Application(tornado.web.Application):
    def __init__(self):
        handlers = [
            (r"/message", MessageHandle),
            (r"/delay_message", DelayMessageHandle),
            (r"/periodic_message", PeriodicMessageHandle),
            (r"/send_to_message", UserMessageHandle),
        ]
        settings = dict(
            static_path=os.path.join(os.path.dirname(__file__), "static"),
        )
        super(Application, self).__init__(handlers, **settings)


class MessageHandle(StatusWrapperMixin, tornado.web.RequestHandler):
    def post(self, *args, **kwargs):
        message = self.get_argument('content', None)
        token = self.get_argument('token', None)
        if message:
            if glb.token:
                if glb.token != token:
                    self.status_code = STATUS_PERMISSION_DENIED
                    self.write('Token is missing')
                    return
            try:
                glb.wxbot.send_msg(message)
                self.write('Success')
            except Exception as e:
                _logger.exception(e)
                self.status_code = STATUS_BOT_EXCEPTION
                self.write(e.message)


class DelayMessageHandle(StatusWrapperMixin, tornado.web.RequestHandler):
    def __init__(self, application, request, *args, **kwargs):
        self.ioloop = tornado.ioloop.IOLoop.instance()
        super(DelayMessageHandle, self).__init__(application, request, *args, **kwargs)

    def post(self, *args, **kwargs):
        content = self.get_argument('content', '')
        title = self.get_argument('title', '')
        task_time = self.get_argument('time', None)
        remind = int(self.get_argument('remind', DEFAULT_REMIND_TIME))
        token = self.get_argument('token', None)
        if glb.token:
            if glb.token != token:
                self.status_code = STATUS_PERMISSION_DENIED
                self.write('Token is missing')
                return
        if task_time:
            try:
                task_time = datetime.datetime.strptime(task_time, '%Y-%m-%d %H:%M:%S')
                timestamp = time.mktime(
                    (task_time - datetime.timedelta(
                        seconds=remind)).timetuple())
            except ValueError as e:
                self.status_code = STATUS_ERROR
                self.write(e.message)
                _logger.exception(e)
                return
        else:
            task_time = datetime.datetime.now()
            timestamp = int(time.mktime(task_time.timetuple()))
        try:
            message = Message(content, title, task_time, datetime.timedelta(seconds=remind))
            self.ioloop.call_at(timestamp, self.delay_task, DELAY_TASK, message)
            self.write('Success')
        except Exception as e:
            self.status_code = STATUS_TORNADO_EXCEPTION
            self.write(e.message)
            _logger.exception(e)

    @staticmethod
    def delay_task(task_type, message):
        # try:
        wxbot.send_msg(message)
        _logger.info(
            '{0} Send delay message {1} at {2:%Y-%m-%d %H:%M:%S}'.format(task_type, message, datetime.datetime.now()))
        # except Exception as e:


class PeriodicMessageHandle(StatusWrapperMixin, tornado.web.RequestHandler):
    def __init__(self, application, request, *args, **kwargs):
        self.ioloop = tornado.ioloop.IOLoop.instance()
        super(PeriodicMessageHandle, self).__init__(application, request, *args, **kwargs)

    def post(self, *args, **kwargs):
        content = self.get_argument('content', '')
        title = self.get_argument('title', '')
        interval = self.get_argument('interval', None)
        token = self.get_argument('token', None)
        if glb.token:
            if glb.token != token:
                self.status_code = STATUS_PERMISSION_DENIED
                self.write('Token is missing')
                return
        if not interval:
            self.status_code = STATUS_ERROR
            self.write('interval is required')
            return
        try:
            interval = int(interval)
        except Exception as e:
            self.status_code = STATUS_ERROR
            self.write('interval must be a integer')
        try:
            message = Message(content, title=title, interval=datetime.timedelta(seconds=interval))
            user_periodic = tornado.ioloop.PeriodicCallback(
                functools.partial(self.periodic_task, PERIODIC_TASK, message),
                interval * 1000, self.ioloop)
            glb.periodic_list.append(user_periodic)
            user_periodic.start()
            self.write('Success')
        except Exception as e:
            self.status_code = STATUS_TORNADO_EXCEPTION
            self.write(e.message)
            _logger.exception(e)

    @staticmethod
    def periodic_task(task_type, message):
        glb.wxbot.send_msg(message)
        _logger.info('{0} Send periodic message {1} at {2:%Y-%m-%d %H:%M:%S}'.format(task_type, message,
                                                                                     datetime.datetime.now()))


class UserMessageHandle(StatusWrapperMixin, tornado.web.RequestHandler):
    def post(self, *args, **kwargs):
        from wxpy import ensure_one

        content = self.get_argument('content', '')
        search = self.get_argument('search', '')
        token = self.get_argument('token', None)
        if glb.token:
            if glb.token != token:
                self.status_code = STATUS_PERMISSION_DENIED
                self.write('Token is missing')
                return
        try:
            search = json.loads(search)
        except ValueError:
            search = search
        try:
            if isinstance(search, dict):
                receiver = ensure_one(glb.wxbot.bot.friends(update=True).search(**search))
            else:
                receiver = ensure_one(glb.wxbot.bot.friends(update=True).search(search))
        except ValueError:
            receiver = None
        if receiver:
            receiver.send_msg(content)
        else:
            msg = '消息发送失败，没有找到接收者。\n[搜索条件]: {0}\n[消息内容]：{1}'.format(search, content)
            glb.wxbot.send_msg(msg)
            _logger.info(msg)
        self.write('Success')


def generate_run_info():
    uptime = datetime.datetime.now() - datetime.datetime.fromtimestamp(glb.run_info.create_time())
    memory_usage = glb.run_info.memory_info().rss
    msg = '[当前时间] {now:%H:%M:%S}\n[运行时间] {uptime}\n[内存占用] {memory}\n[发送消息] {messages}'.format(
        now=datetime.datetime.now(),
        uptime=str(uptime).split('.')[0],
        memory='{:.2f} MB'.format(memory_usage / 1024 ** 2),
        messages=len(glb.wxbot.bot.messages)
    )
    return msg


def check_bot(task_type=SYSTEM_TASK):
    if glb.wxbot.bot.alive:
        msg = generate_run_info()
        glb.wxbot.send_msg(msg)
        _logger.info(
            '{0} Send status message {1} at {2:%Y-%m-%d %H:%M:%S}'.format(task_type, msg, datetime.datetime.now()))
    else:
        # todo
        pass


def timeout_message_report():
    timeout_list = glb.ioloop._timeouts
    delay_task = []
    for timeout in timeout_list:
        if not timeout.callback:
            continue
        if len(timeout.callback.args) == 2:
            task_type, message = timeout.callback.args
            delay_task.append(message)
    msg = '当前已注册延时消息共有{0}条'.format(len(delay_task))
    for i, itm in enumerate(delay_task):
        msg = '{pre}\n[ID (序号) ]：D{index}\n[发送时间]：{remind}\n[消息时间]：{time}\n[消息标题]：{message}\n'.format(
            pre=msg, index=i, remind=itm.remind, time=itm.time, message=itm.title or itm.content)
    interval_task = [(periodic.callback.args[1], periodic.is_running()) for periodic in glb.periodic_list if
                     len(periodic.callback.args) == 2 and periodic.callback.args[0] == PERIODIC_TASK]
    msg = '{0}\n当前已注册周期消息共有{1}条'.format(msg, len(interval_task))
    for i, itm in enumerate(interval_task):
        msg = '{pre}\n[ID (序号) ]：P{index}\n[运行状态]：{status}\n[发送周期]：{interval}\n[消息标题]：{message}\n'.format(
            pre=msg, index=i, interval=itm[0].interval, status='已激活' if itm[1] else '未激活',
            message=itm[0].title or itm[0].content)
    return msg


def register_listener_handle(wxbot):
    from wxpy import TEXT

    @wxbot.bot.register(wxbot.receiver, TEXT, except_self=False)
    def sender_command_handle(msg):
        command_dict = {MESSAGE_REPORT_COMMAND: timeout_message_report(),
                        MESSAGE_STATUS_COMMAND: generate_run_info()}
        message = command_dict.get(msg.text, None)
        if message:
            return message
        myself = wxbot.bot.registered.get_config(msg)
        registered_copy = copy.copy(wxbot.bot.registered)
        registered_copy.remove(myself)
        pre_conf = registered_copy.get_config(msg)
        if pre_conf:
            my_name = sys._getframe().f_code.co_name
            if my_name != pre_conf.func.__name__:
                pre_conf.func(msg)


def listen(bot, receiver=None, token=None, port=10245, status_report=False):
    global glb
    periodic_list = []
    app = Application()
    wxbot = WxBot(bot, receiver)
    register_listener_handle(wxbot)
    process = psutil.Process()
    app.listen(port)

    if status_report:
        check_periodic = tornado.ioloop.PeriodicCallback(functools.partial(check_bot, SYSTEM_TASK), DEFAULT_REPORT_TIME)
        check_periodic.start()
        periodic_list.append(check_periodic)

    glb = Global(wxbot=wxbot, run_info=process, periodic_list=periodic_list, ioloop=tornado.ioloop.IOLoop.instance(),
                 token=token)
    tornado.ioloop.IOLoop.current().start()



# coding: utf-8
from __future__ import unicode_literals

import json

import datetime
import requests

from wechat_sender.utils import STATUS_SUCCESS, DEFAULT_REMIND_TIME


class Sender(object):
    def __init__(self, token=None, host='http://localhost', port=10245):
        self.token = token
        self.host = host
        self.port = port
        self.remote = '{0}:{1}/'.format(self.host, self.port)
        self.data = {}

    def _wrap_post_data(self, **kwargs):
        self.data = kwargs
        if self.token:
            self.data['token'] = self.token
        return self.data

    def send(self, message):
        url = '{0}message'.format(self.remote)
        data = self._wrap_post_data(content=message)
        res = requests.post(url, data=data, timeout=2)
        if res.status_code == requests.codes.ok:
            res_data = json.loads(res.content)
            if res_data.get('status') == STATUS_SUCCESS:
                return True, res_data.get('message')
            return False, res_data.get('message')
        res.raise_for_status()
        return False, 'Request or Response Error'

    def delay_send(self, content, time, title='', remind=DEFAULT_REMIND_TIME):
        url = '{0}delay_message'.format(self.remote)
        if isinstance(time, (datetime.datetime, datetime.date)):
            time = time.strftime('%Y-%m-%d %H:%M:%S')
        if isinstance(remind, datetime.timedelta):
            remind = remind.seconds
        if not isinstance(remind, (int, long)):
            raise ValueError
        data = self._wrap_post_data(title=title, content=content, time=time, remind=remind)
        res = requests.post(url, data=data, timeout=2)
        if res.status_code == requests.codes.ok:
            res_data = json.loads(res.content)
            if res_data.get('status') == STATUS_SUCCESS:
                return True, res_data.get('message')
            return False, res_data.get('message')
        res.raise_for_status()
        return False, 'Request or Response Error'

    def periodic_send(self, content, interval, title=''):
        url = '{0}periodic_message'.format(self.remote)
        if isinstance(interval, datetime.timedelta):
            interval = interval.seconds
        if not isinstance(interval, (int, long)):
            raise ValueError
        data = self._wrap_post_data(title=title, content=content, interval=interval)
        res = requests.post(url, data, timeout=2)
        if res.status_code == requests.codes.ok:
            res_data = json.loads(res.content)
            if res_data.get('status') == STATUS_SUCCESS:
                return True, res_data.get('message')
            return False, res_data.get('message')
        res.raise_for_status()
        return False, 'Request or Response Error'

    def send_to(self, content, search):
        url = '{0}send_to_message'.format(self.remote)
        if isinstance(search, dict):
            search = json.dumps(search)
        elif isinstance(search, list):
            search = reduce(lambda x, y: '{0} {1}'.format(x, y), search)
        data = self._wrap_post_data(content=content, search=search)
        res = requests.post(url, data=data, timeout=2)
        if res.status_code == requests.codes.ok:
            res_data = json.loads(res.content)
            if res_data.get('status') == STATUS_SUCCESS:
                return True, res_data.get('message')
            return False, res_data.get('message')
        res.raise_for_status()
        return False, 'Request or Response Error'